package envoy.client.ui;

import java.awt.*;
import java.lang.reflect.Constructor;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JPanel;

import envoy.client.Settings;
import envoy.client.event.Event;
import envoy.client.event.EventBus;
import envoy.client.util.EnvoyLog;

/**
 * This Component can be used to toggle between two options. e.g. on and
 * off.<br>
 * <br>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>PrimaryToggleSwitch.java</strong><br>
 * Created: <strong>21 Dec 2019</strong><br>
 * 
 * @author Maximilian K&auml;fer
 * @since Envoy v0.3-alpha
 */
public class PrimaryToggleSwitch extends JPanel {

	private final JButton b = new JButton("");

	private boolean currentState;

	private static final Logger	logger				= EnvoyLog.getLogger(PrimaryToggleSwitch.class.getSimpleName());
	private static final long	serialVersionUID	= -721155303106833184L;

	/**
	 * This is the constructor for the PrimaryToggleSwitch.
	 * 
	 * @param initialState The state the toggleSwitch is standardly set to. </br>
	 *                     true: off </br>
	 *                     false: on
	 * @param eventClass   the class of the event dispatched by this toggleSwitch
	 * @since Envoy v0.3-alpha
	 */
	public PrimaryToggleSwitch(boolean initialState, Class<? extends Event<Boolean>> eventClass) {
		setEnabled(true);
		setVisible(true);

		setPreferredSize(new Dimension(50, 25));
		setMinimumSize(new Dimension(50, 25));
		setMaximumSize(new Dimension(50, 25));

		b.setPreferredSize(new Dimension(25, 25));
		b.setMinimumSize(new Dimension(25, 25));
		b.setMaximumSize(new Dimension(25, 25));

		b.setBackground(Settings.getInstance().getThemes().get(Settings.getInstance().getCurrentTheme()).getInteractableBackgroundColor());

		GridBagLayout gbl_toggleSwitch = new GridBagLayout();
		gbl_toggleSwitch.columnWidths	= new int[] { 1, 1 };
		gbl_toggleSwitch.rowHeights		= new int[] { 1 };
		gbl_toggleSwitch.columnWeights	= new double[] { 1.0, 1.0 };
		gbl_toggleSwitch.rowWeights		= new double[] { 1.0 };

		setLayout(gbl_toggleSwitch);

		setState(initialState);

		b.addActionListener((evt) -> {
			try {
				// Dispatch event
				Constructor<? extends Event<Boolean>> constructor = eventClass.getConstructor(boolean.class);
				EventBus.getInstance().dispatch(constructor.newInstance(currentState));

				setState(!currentState);
				revalidate();
				repaint();
			} catch (ReflectiveOperationException | SecurityException e) {
				logger.warning("An error occured while changing the setting: " + e);
			}
		});

		repaint();
	}

	@Override
	public void paintComponent(Graphics g) {
		g.setColor(Color.LIGHT_GRAY);
		g.fillRect(0, 0, 50, 25);
		g.setColor(Color.GREEN);
		g.fillRect(0, 0, 25, 25);
	}

	/**
	 * This method sets the state of this {@link PrimaryToggleSwitch}.
	 * 
	 * @param state {@code true} to enable the switch, {@code false} to disable it
	 * @since Envoy 0.3-alpha
	 */
	public void setState(boolean state) {
		GridBagConstraints gbc_toggleButton = new GridBagConstraints();

		if (state) {
			gbc_toggleButton.anchor	= GridBagConstraints.WEST;
			gbc_toggleButton.gridx	= 0;
		} else {
			gbc_toggleButton.anchor	= GridBagConstraints.EAST;
			gbc_toggleButton.gridx	= 1;
		}
		gbc_toggleButton.gridy = 0;
		add(b, gbc_toggleButton);

		currentState = state;
	}
}
